@@ -42,6 +42,8 @@ SOFTWARE.
 #define BLAS_INT int
 #endif
 
+double *logGammaArr2 = NULL;
+
 /**
  * @brief Computes the value of `r` without the denominator.
  *
@@ -68,7 +70,8 @@ double computeR(Matrix const *probabilities, Matrix const *mult, int const b, in
  * to compute the lgamma function
  *
  * This would be called outside the computing function, so it is not designed to "save" some calculations.
-static void precomputeLogGammas()
+ */
+static void precomputeLogGammasMult()
 {
     // We must get the biggest W_{bg}
     int biggestB = 0;
@@ -83,7 +86,6 @@ static void precomputeLogGammas()
         logGammaArr2[i] = lgamma1p(i);
     }
 }
-*/
 /**
  * @brief Computes an approximate of the conditional probability by using a Multinomial approach.
  *
@@ -101,10 +103,10 @@ static void precomputeLogGammas()
 double *computeQMultinomial(Matrix const *probabilities, QMethodInput params, double *ll)
 {
 
-    // if (logGammaArr2 == NULL)
-    //{
-    // precomputeLogGammas();
-    //}
+    if (logGammaArr2 == NULL)
+    {
+        precomputeLogGammasMult();
+    }
     double *array2 = (double *)Calloc(TOTAL_BALLOTS * TOTAL_CANDIDATES * TOTAL_GROUPS, double); // Array to return
     *ll = 0;
     // -- Summatory calculation for g --
@@ -188,7 +190,6 @@ double *computeQMultinomial(Matrix const *probabilities, QMethodInput params, do
     return array2;
 }
 
-/*
 void cleanMultinomial(void)
 {
     if (logGammaArr2 != NULL)
@@ -197,4 +198,3 @@ void cleanMultinomial(void)
         logGammaArr2 = NULL;
     }
 }
-*/
